/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */
package ink.xcl.leventbus.core

import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.os.Handler
import android.os.Looper
import androidx.annotation.MainThread
import androidx.lifecycle.ExternalLiveData
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.LiveData
import androidx.lifecycle.Observer
import ink.xcl.leventbus.ipc.consts.IpcConst
import ink.xcl.leventbus.ipc.core.ProcessorManager
import ink.xcl.leventbus.ipc.receiver.LebIpcReceiver
import ink.xcl.leventbus.logger.DefaultLogger
import ink.xcl.leventbus.logger.Logger
import ink.xcl.leventbus.logger.LoggerManager
import ink.xcl.leventbus.utils.AppUtils.Companion.app
import ink.xcl.leventbus.utils.ThreadUtils.isMainThread
import java.util.logging.Level

/** LiveEventBusCore */
class LiveEventBusCore private constructor() {
    /** 单例模式实现 */
    private object SingletonHolder {
        val DEFAULT_BUS = LiveEventBusCore()
    }

    /** 存放LiveEvent */
    private val bus: MutableMap<String, LiveEvent<Any>>

    /** 可配置的项 */
    private val config = Config()
    private var lifecycleObserverAlwaysActive = true
    private var autoClear = false
    private val logger: LoggerManager
    private val observableConfigs: MutableMap<String, ObservableConfig>

    /** 跨进程通信 */
    private val receiver: LebIpcReceiver
    private var isRegisterReceiver = false

    /** 调试 */
    val console = InnerConsole()

    init {
        bus = HashMap()
        observableConfigs = HashMap()
        logger = LoggerManager(DefaultLogger())
        receiver = LebIpcReceiver()
        registerReceiver()
    }

    @Suppress("UNCHECKED_CAST")
    @Synchronized
    fun <T> with(key: String, type: Class<T>?): Observable<T> {
        if (!bus.containsKey(key)) {
            bus[key] = LiveEvent(key)
        }
        return bus[key] as Observable<T>
    }

    /**
     * use the class Config to set params first of all, call config to get the Config instance then,
     * call the method of Config to config LiveEventBus call this method in Application.onCreate
     *
     * @return Config
     */
    fun config(): Config {
        return config
    }

    fun config(key: String): ObservableConfig? {
        if (!observableConfigs.containsKey(key)) {
            observableConfigs[key] = ObservableConfig()
        }
        return observableConfigs[key]
    }

    fun setLogger(logger: Logger) {
        this.logger.logger = logger
    }

    fun enableLogger(enable: Boolean) {
        logger.isEnable = enable
    }

    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    fun registerReceiver() {
        if (isRegisterReceiver) {
            return
        }
        val application = app
        if (application != null) {
            val intentFilter = IntentFilter()
            intentFilter.addAction(IpcConst.ACTION)
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                application.registerReceiver(receiver, intentFilter, Context.RECEIVER_EXPORTED)
            } else {
                application.registerReceiver(receiver, intentFilter)
            }
            isRegisterReceiver = true
        }
    }

    fun setLifecycleObserverAlwaysActive(lifecycleObserverAlwaysActive: Boolean) {
        this.lifecycleObserverAlwaysActive = lifecycleObserverAlwaysActive
    }

    fun setAutoClear(autoClear: Boolean) {
        this.autoClear = autoClear
    }

    private inner class LiveEvent<T : Any>(private val key: String) : Observable<T> {
        val liveData: LifecycleLiveData<T> = LifecycleLiveData(key)
        val observerMap: MutableMap<Observer<*>, ObserverWrapper<T>> = HashMap()
        val mainHandler = Handler(Looper.getMainLooper())

        /**
         * 进程内发送消息
         *
         * @param value 发送的消息
         */
        override fun post(value: T) {
            if (isMainThread) {
                postInternal(value)
            } else {
                mainHandler.post(PostValueTask(value))
            }
        }

        /**
         * App内发送消息，跨进程使用
         *
         * @param value 发送的消息
         */
        override fun postAcrossProcess(value: T) {
            broadcast(value, foreground = false, onlyInApp = true)
        }

        /**
         * App之间发送消息
         *
         * @param value 发送的消息
         */
        override fun postAcrossApp(value: T) {
            broadcast(value, foreground = false, onlyInApp = false)
        }

        /**
         * 进程内发送消息，延迟发送
         *
         * @param value 发送的消息
         * @param delay 延迟毫秒数
         */
        override fun postDelay(value: T, delay: Long) {
            mainHandler.postDelayed(PostValueTask(value), delay)
        }

        /**
         * 进程内发送消息，延迟发送，带生命周期 如果延时发送消息的时候sender处于非激活状态，消息取消发送
         *
         * @param sender 消息发送者
         * @param value 发送的消息
         * @param delay 延迟毫秒数
         */
        override fun postDelay(sender: LifecycleOwner?, value: T, delay: Long) {
            mainHandler.postDelayed(PostLifeValueTask(value, sender), delay)
        }

        /**
         * 进程内发送消息 强制接收到消息的顺序和发送顺序一致
         *
         * @param value 发送的消息
         */
        override fun postOrderly(value: T) {
            mainHandler.post(PostValueTask(value))
        }

        /**
         * App之间发送消息
         *
         * @param value 发送的消息
         */
        @Deprecated("", ReplaceWith("broadcast(value, false, onlyInApp = false)"))
        override fun broadcast(value: T) {
            broadcast(value, false, onlyInApp = false)
        }

        /**
         * 以广播的形式发送一个消息 需要跨进程、跨APP发送消息的时候调用该方法
         *
         * @param value 发送的消息
         * @param foreground true:前台广播、false:后台广播
         * @param onlyInApp true:只在APP内有效、false:全局有效
         */
        override fun broadcast(value: T, foreground: Boolean, onlyInApp: Boolean) {
            if (app != null) {
                if (isMainThread) {
                    broadcastInternal(value, foreground, onlyInApp)
                } else {
                    mainHandler.post { broadcastInternal(value, foreground, onlyInApp) }
                }
            } else {
                post(value)
            }
        }

        /**
         * 注册一个Observer，生命周期感知，自动取消订阅
         *
         * @param owner LifecycleOwner
         * @param observer 观察者
         */
        override fun observe(owner: LifecycleOwner, observer: Observer<T>) {
            if (isMainThread) {
                observeInternal(owner, observer)
            } else {
                mainHandler.post { observeInternal(owner, observer) }
            }
        }

        /**
         * 注册一个Observer，生命周期感知，自动取消订阅 如果之前有消息发送，可以在注册时收到消息（消息同步）
         *
         * @param owner LifecycleOwner
         * @param observer 观察者
         */
        override fun observeSticky(owner: LifecycleOwner, observer: Observer<T>) {
            if (isMainThread) {
                observeStickyInternal(owner, observer)
            } else {
                mainHandler.post { observeStickyInternal(owner, observer) }
            }
        }

        /**
         * 注册一个Observer，需手动解除绑定
         *
         * @param observer 观察者
         */
        override fun observeForever(observer: Observer<T>) {
            if (isMainThread) {
                observeForeverInternal(observer)
            } else {
                mainHandler.post { observeForeverInternal(observer) }
            }
        }

        /**
         * 注册一个Observer，需手动解除绑定 如果之前有消息发送，可以在注册时收到消息（消息同步）
         *
         * @param observer 观察者
         */
        override fun observeStickyForever(observer: Observer<T>) {
            if (isMainThread) {
                observeStickyForeverInternal(observer)
            } else {
                mainHandler.post { observeStickyForeverInternal(observer) }
            }
        }

        /**
         * 通过observeForever或observeStickyForever注册的，需要调用该方法取消订阅
         *
         * @param observer 观察者
         */
        override fun removeObserver(observer: Observer<T>) {
            if (isMainThread) {
                removeObserverInternal(observer)
            } else {
                mainHandler.post { removeObserverInternal(observer) }
            }
        }

        @MainThread
        private fun postInternal(value: T) {
            logger.log(Level.INFO, "post: $value with key: $key")
            liveData.value = value
        }

        @MainThread
        private fun broadcastInternal(value: T, foreground: Boolean, onlyInApp: Boolean) {
            logger.log(Level.INFO, "broadcast: $value foreground: $foreground with key: $key")
            val application = app
            if (application == null) {
                logger.log(
                    Level.WARNING, "application is null, you can try setContext() when config")
                return
            }
            val intent = Intent(IpcConst.ACTION)
            if (foreground) {
                intent.addFlags(Intent.FLAG_RECEIVER_FOREGROUND)
            }
            if (onlyInApp) {
                intent.setPackage(application.packageName)
            }
            intent.putExtra(IpcConst.KEY, key)
            val handle = ProcessorManager.getManager().writeTo(intent, value)
            try {
                if (handle) {
                    application.sendBroadcast(intent)
                }
            } catch (e: Exception) {
                logger.log(Level.WARNING, "Err=$e")
            }
        }

        @MainThread
        private fun observeInternal(owner: LifecycleOwner, observer: Observer<T>) {
            val observerWrapper: ObserverWrapper<T> = ObserverWrapper(observer)
            observerWrapper.preventNextEvent = liveData.version > ExternalLiveData.START_VERSION
            liveData.observe(owner, observerWrapper)
            logger.log(
                Level.INFO,
                "observe observer: " +
                    observerWrapper +
                    "(" +
                    observer +
                    ")" +
                    " on owner: " +
                    owner +
                    " with key: " +
                    key)
        }

        @MainThread
        private fun observeStickyInternal(owner: LifecycleOwner, observer: Observer<T>) {
            val observerWrapper: ObserverWrapper<T> = ObserverWrapper(observer)
            liveData.observe(owner, observerWrapper)
            logger.log(
                Level.INFO,
                "observe sticky observer: " +
                    observerWrapper +
                    "(" +
                    observer +
                    ")" +
                    " on owner: " +
                    owner +
                    " with key: " +
                    key)
        }

        @MainThread
        private fun observeForeverInternal(observer: Observer<T>) {
            val observerWrapper: ObserverWrapper<T> = ObserverWrapper(observer)
            observerWrapper.preventNextEvent = liveData.version > ExternalLiveData.START_VERSION
            observerMap[observer] = observerWrapper
            liveData.observeForever(observerWrapper)
            logger.log(
                Level.INFO,
                "observe forever observer: " +
                    observerWrapper +
                    "(" +
                    observer +
                    ")" +
                    " with key: " +
                    key)
        }

        @MainThread
        private fun observeStickyForeverInternal(observer: Observer<T>) {
            val observerWrapper: ObserverWrapper<T> = ObserverWrapper(observer)
            observerMap[observer] = observerWrapper
            liveData.observeForever(observerWrapper)
            logger.log(
                Level.INFO,
                "observe sticky forever observer: " +
                    observerWrapper +
                    "(" +
                    observer +
                    ")" +
                    " with key: " +
                    key)
        }

        @MainThread
        private fun removeObserverInternal(observer: Observer<T>) {
            val realObserver: Observer<T>? =
                if (observerMap.containsKey(observer)) {
                    observerMap.remove(observer)
                } else {
                    observer
                }
            if (realObserver != null) {
                liveData.removeObserver(realObserver)
            }
        }

        inner class LifecycleLiveData<T>(private val key: String) : ExternalLiveData<T>() {
            override fun observerActiveLevel(): Lifecycle.State {
                return if (lifecycleObserverAlwaysActive() == true) Lifecycle.State.CREATED
                else Lifecycle.State.STARTED
            }

            override fun removeObserver(observer: Observer<in T>) {
                super.removeObserver(observer)
                if (autoClear() == true && !liveData.hasObservers()) {
                    get().bus.remove(key)
                }
                logger.log(Level.INFO, "observer removed: $observer")
            }

            private fun lifecycleObserverAlwaysActive(): Boolean? {
                if (observableConfigs.containsKey(key)) {
                    val config = observableConfigs[key]
                    if (config?.lifecycleObserverAlwaysActive != null) {
                        return config.lifecycleObserverAlwaysActive
                    }
                }
                return lifecycleObserverAlwaysActive
            }

            private fun autoClear(): Boolean? {
                if (observableConfigs.containsKey(key)) {
                    val config = observableConfigs[key]
                    if (config?.autoClear != null) {
                        return config.autoClear
                    }
                }
                return autoClear
            }
        }

        private inner class PostValueTask(private val newValue: Any) : Runnable {
            @Suppress("UNCHECKED_CAST")
            override fun run() {
                postInternal(newValue as T)
            }
        }

        private inner class PostLifeValueTask(
            private val newValue: Any,
            private val owner: LifecycleOwner?
        ) : Runnable {
            @Suppress("UNCHECKED_CAST")
            override fun run() {
                if (owner != null) {
                    if (owner.lifecycle.currentState.isAtLeast(Lifecycle.State.STARTED)) {
                        postInternal(newValue as T)
                    }
                }
            }
        }
    }

    private inner class ObserverWrapper<T>(observer: Observer<T>) : Observer<T> {
        val observer: Observer<T>
        var preventNextEvent = false

        init {
            this.observer = observer
        }

        override fun onChanged(value: T) {
            if (preventNextEvent) {
                preventNextEvent = false
                return
            }
            logger.log(Level.INFO, "message received: $value")
            try {
                observer.onChanged(value)
            } catch (e: ClassCastException) {
                logger.log(Level.WARNING, "class cast error on message received: $value", e)
            } catch (e: Exception) {
                logger.log(Level.WARNING, "error on message received: $value", e)
            }
        }
    }

    inner class InnerConsole {
        val consoleInfo: String
            get() =
                """
                 *********Base info*********
                 ${baseInfo}*********Event info*********
                 $busInfo
                 """
                    .trimIndent()

        private val baseInfo: String
            get() =
                """
                 lifecycleObserverAlwaysActive: $lifecycleObserverAlwaysActive
                 autoClear: $autoClear
                 logger enable: ${logger.isEnable}
                 logger: ${logger.logger}
                 Receiver register: $isRegisterReceiver
                 Application: $app
                 
                 """
                    .trimIndent()

        private val busInfo: String
            get() {
                val sb = StringBuilder()
                for (key in bus.keys) {
                    sb.append("Event name: ").append(key).append("\n")
                    val liveData: LiveEvent<Any>.LifecycleLiveData<Any>? = bus[key]?.liveData
                    liveData?.let {
                        sb.append("\tversion: ").append(liveData.version).append("\n")
                        sb.append("\thasActiveObservers: ")
                            .append(liveData.hasActiveObservers())
                            .append("\n")
                        sb.append("\thasObservers: ").append(liveData.hasObservers()).append("\n")
                        sb.append("\tActiveCount: ").append(getActiveCount(liveData)).append("\n")
                        sb.append("\tObserverCount: ")
                            .append(getObserverCount(liveData))
                            .append("\n")
                        sb.append("\tObservers: ").append("\n")
                        sb.append("\t\t" + getObserverInfo(liveData)).append("\n")
                    }
                }
                return sb.toString()
            }

        private fun getActiveCount(liveData: LiveData<*>): Int {
            return try {
                val field = LiveData::class.java.getDeclaredField("mActiveCount")
                field.isAccessible = true
                field[liveData] as Int
            } catch (e: Exception) {
                -1
            }
        }

        private fun getObserverCount(liveData: LiveData<*>): Int {
            return try {
                val field = LiveData::class.java.getDeclaredField("mObservers")
                field.isAccessible = true
                val mObservers = field[liveData]
                val classOfSafeIterableMap: Class<*> = mObservers.javaClass
                val size = classOfSafeIterableMap.getDeclaredMethod("size")
                size.isAccessible = true
                size.invoke(mObservers) as Int
            } catch (e: Exception) {
                -1
            }
        }

        private fun getObserverInfo(liveData: LiveData<*>): String? {
            return try {
                val field = LiveData::class.java.getDeclaredField("mObservers")
                field.isAccessible = true
                val mObservers = field[liveData]
                mObservers?.toString()
            } catch (e: Exception) {
                ""
            }
        }
    }

    companion object {
        fun get(): LiveEventBusCore {
            return SingletonHolder.DEFAULT_BUS
        }
    }
}
